@@ -7,6 +7,8 @@ import numpy as np
 from numpy.linalg import norm
 import cvxpy as cvx
 from collections import defaultdict
+from statistical_clear_sky.algorithm.time_shift.clustering\
+import ClusteringTimeShift
 from\
  statistical_clear_sky.algorithm.initialization.singular_value_decomposition\
  import SingularValueDecomposition
@@ -24,7 +26,7 @@ from statistical_clear_sky.algorithm.serialization.serialization_mixin\
  import SerializationMixin
 from statistical_clear_sky.algorithm.plot.plot_mixin import PlotMixin
 from statistical_clear_sky.utilities.data_loading import resample_index
-from statistical_clear_sky.utilities.progress import progress
+from solardatatools.utilities import progress
 
 class IterativeFitting(SerializationMixin, PlotMixin):
     """
@@ -32,7 +34,8 @@ class IterativeFitting(SerializationMixin, PlotMixin):
     """
 
     def __init__(self, data_matrix=None, data_handler_obj=None, rank_k=6,
-                 solver_type='MOSEK', reserve_test_data=False):
+                 solver_type='MOSEK', reserve_test_data=False,
+                 auto_fix_time_shifts=False, time_shift=None):
         """
 
         :param data_matrix:
@@ -40,6 +43,8 @@ class IterativeFitting(SerializationMixin, PlotMixin):
         :param rank_k:
         :param solver_type:
         :param reserve_test_data:
+        :param auto_fix_time_shifts:
+        :param time_shift:
         """
         self._solver_type = solver_type
         self._rank_k = rank_k
@@ -48,12 +53,12 @@ class IterativeFitting(SerializationMixin, PlotMixin):
         elif data_handler_obj is not None:
             data_matrix = data_handler_obj.filled_data_matrix
             self._power_signals_d = data_matrix
-            self._capacity = data_handler_obj.capacity_estimate
             # Set the weighting now, to use the error flagging feature
             weights = self._get_weight_setting().obtain_weights(data_matrix)
             weights *= data_handler_obj.daily_flags.no_errors
         else:
-            self._power_signals_d = data_matrix
+            self._power_signals_d = self._handle_time_shift(
+                data_matrix, auto_fix_time_shifts, time_shift=time_shift)
 
         self._decomposition = SingularValueDecomposition()
         self._decomposition.decompose(data_matrix, rank_k=rank_k)
@@ -69,7 +74,7 @@ class IterativeFitting(SerializationMixin, PlotMixin):
 
         # Stores the current state of the object:
         self._state_data = StateData()
-        self._store_initial_state_data()
+        self._store_initial_state_data(auto_fix_time_shifts)
         if data_handler_obj is not None:
             self._weights = weights
             self._state_data.weights = weights
@@ -107,40 +112,9 @@ class IterativeFitting(SerializationMixin, PlotMixin):
             sum_components=sum_components)
 
     @property
-    def measured_power_matrix(self):
+    def power_signals_d(self):
         return self._power_signals_d
 
-    @property
-    def estimated_power_matrix(self):
-        left = self._l_cs_value
-        right = self._r_cs_value
-        mat = left.dot(right)
-        return mat
-
-    @property
-    def estimated_clear_sky(self):
-        return self._l_cs_value @ self._r_cs_value
-
-    @property
-    def deg_rate(self):
-        return self._beta_value.item()
-
-    @property
-    def left_matrix(self):
-        return self._l_cs_value
-
-    @property
-    def right_matrix(self):
-        return self._r_cs_value
-
-    @property
-    def left_problem(self):
-        return self._l_problem
-
-    @property
-    def right_problem(self):
-        return self._r_problem
-
     @property
     def l_cs_value(self):
         return self._l_cs_value
@@ -215,47 +189,11 @@ class IterativeFitting(SerializationMixin, PlotMixin):
             is_degradation_calculated=is_degradation_calculated,
             max_degradation=max_degradation,
             min_degradation=min_degradation)
-
-        self._l_problem = left_matrix_minimization
-        self._r_problem = right_matrix_minimization
         ti = time()
         objective_values = self._calculate_objective(mu_l, mu_r, tau,
             l_cs_value, r_cs_value, beta_value, weights,
             sum_components=False)
         if verbose:
-            print('----------------------\nSCSF Problem Setup\n----------------------')
-            msg1 = 'Matrix Size: {} x {} = {} power measurements'.format(
-                self._power_signals_d.shape[0],
-                self._power_signals_d.shape[1],
-                self._power_signals_d.size
-            )
-            print(msg1)
-            reduced_mat = self._power_signals_d[:, self._weights > 0]
-            try:
-                real_meas = reduced_mat > 0.005 * self._capacity
-            except:
-                real_meas = reduced_mat > 0.005 * np.nanquantile(self._power_signals_d, 0.95)
-            msg = 'Sparsity: {:.2f}%'.format(
-                100 * (1 - np.sum(real_meas) / self._power_signals_d.size)
-            )
-            print(msg)
-            msg = '{} non-zero measurements under clear conditions'.format(
-                np.sum(real_meas)
-            )
-            print(msg)
-            msg2 = 'Model size: {} x {} + {} x {} = {} parameters'.format(
-                l_cs_value.shape[0],
-                l_cs_value.shape[1],
-                r_cs_value.shape[0],
-                r_cs_value.shape[1],
-                np.sum([
-                    l_cs_value.shape[0] * l_cs_value.shape[1],
-                    r_cs_value.shape[0] * r_cs_value.shape[1]
-                ])
-            )
-            print(msg2)
-            print('\n')
-            print('----------------------\nAlgorithm Iterations\n----------------------')
             ps = 'Starting at Objective: {:.3e}, f1: {:.3e}, f2: {:.3e},'
             ps += ' f3: {:.3e}, f4: {:.3e}'
             print(ps.format(
@@ -603,7 +541,7 @@ class IterativeFitting(SerializationMixin, PlotMixin):
         term_f1 = (cvx.sum((0.5 * cvx.abs(
                     self._power_signals_d - l_cs_value.dot(r_cs_value))
                     + (tau - 0.5) * (self._power_signals_d - l_cs_value.dot(
-                        r_cs_value))) @ weights_w1) ).value
+                        r_cs_value))) * weights_w1)).value
         weights_w2 = np.eye(self._rank_k)
         term_f2 = mu_l * norm((l_cs_value[:-2, :] - 2 * l_cs_value[1:-1, :] +
                                l_cs_value[2:, :]).dot(weights_w2), 'fro')
@@ -625,6 +563,19 @@ class IterativeFitting(SerializationMixin, PlotMixin):
         else:
             return components
 
+    def _handle_time_shift(self, power_signals_d, auto_fix_time_shifts,
+                           time_shift=None):
+        self._fixed_time_stamps = False
+        if auto_fix_time_shifts:
+            power_signals_d_fix = self._get_time_shift(
+                power_signals_d, time_shift=time_shift).fix_time_shifts()
+            if np.alltrue(np.isclose(power_signals_d, power_signals_d_fix)):
+                del power_signals_d_fix
+            else:
+                power_signals_d = power_signals_d_fix
+                self._fixed_time_stamps = True
+        return power_signals_d
+
     def _obtain_hyper_parameters(self, mu_l, mu_r, tau):
         if mu_l is None and self._state_data.mu_l is not None:
             mu_l = self._state_data.mu_l
@@ -653,21 +604,17 @@ class IterativeFitting(SerializationMixin, PlotMixin):
 
     def _obtain_initial_component_r0(self, verbose=True):
         if verbose:
-            # print('obtaining initial value of component r0')
-            pass
+            print('obtaining initial value of component r0')
         if self._state_data.component_r0.size > 0:
-            # component_r0 = self._state_data.component_r0
-            component_r0 = np.ones(self._decomposition.matrix_r0.shape[1])
+            component_r0 = self._state_data.component_r0
         else:
-            # component_r0 = self._get_linearization_helper().obtain_component_r0(
-            #     self._decomposition.matrix_r0, index_set=self.weights > 1e-3)
-            component_r0 = np.ones(self._decomposition.matrix_r0.shape[1])
+            component_r0 = self._get_linearization_helper().obtain_component_r0(
+                self._decomposition.matrix_r0, index_set=self.weights > 1e-3)
         return component_r0
 
     def _obtain_weights(self, verbose=True):
         if verbose:
-            # print('obtaining weights')
-            pass
+            print('obtaining weights')
         if self._state_data.weights.size > 0:
             weights = self._state_data.weights
         else:
@@ -717,6 +664,20 @@ class IterativeFitting(SerializationMixin, PlotMixin):
         self._residual_l0_norm = np.linalg.norm(
                 self._matrix_l0[:, 0] - l_cs_value[:, 0])
 
+    def _get_time_shift(self, power_signals_d, time_shift=None):
+        """
+        Method in order to define which TimeShift to use.
+
+        This also works for dependency injection for testing,
+        i.e. for injecting mock.
+        Since it's used in constructor,
+        TimeShift is injected through constructor.
+        """
+        if time_shift is None:
+            return ClusteringTimeShift(power_signals_d)
+        else:
+            return time_shift
+
     def _get_linearization_helper(self):
         """
         For dependency injection for testing, i.e. for injecting mock.
@@ -801,7 +762,8 @@ class IterativeFitting(SerializationMixin, PlotMixin):
     def _keep_supporting_parameters_as_properties(self, weights):
         self._weights = weights
 
-    def _store_initial_state_data(self):
+    def _store_initial_state_data(self, auto_fix_time_shifts):
+        self._state_data.auto_fix_time_shifts = auto_fix_time_shifts
         self._state_data.power_signals_d = self._power_signals_d
         self._state_data.rank_k = self._rank_k
         self._state_data.matrix_l0 = self._matrix_l0
